var cheerio = require('cheerio'),       // jQuery on the server
    fs = require('fs'),                 // Reads the file system
    gutil = require('gulp-util'),       // Utility functions for gulp plugins
    handlebars = require('handlebars'), // HTML templating
    path = require('path'),             // Utility library for handling paths
    through = require('through2'),      // Create streams easily
    _ = require('lodash-node');         // JS utility library

function gulpSVGExample(pluginOptions) {
    var pluginName = 'NP SVG Example',
        defaults = {
        spritePath: '',
		exampleName: 'demo'
    };
    
    pluginOptions = _.merge(defaults, pluginOptions);
    
    // can't find sprite path, exit now
    if (!fs.existsSync(pluginOptions.spritePath) || pluginOptions.spritePath === '') {
        new gutil.PluginError({
          plugin: pluginName,
          message:'Sprite path "' + pluginOptions.spritePath + '" cannot be found.'
        });
    }
  
    return through.obj(function(file, encoding, cb) {
        // Pass file through if:
        // - file has no contents
        // - file is a directory
        if (file.isNull() || file.isDirectory()) {
            this.push(file);
            return cb();
        }
        
        // User's should be using a compatible glob with plugin.
        if (['.svg'].indexOf(path.extname(file.path)) === -1) {
            this.emit('error', new gutil.PluginError({
                plugin: pluginName,
                message: 'Supported format is SVG only.'
            }));
            return cb();
        }
        
        // No support for streams
        if (file.isStream()) {
            this.emit('error', new gutil.PluginError({
                plugin: pluginName,
                message: 'Streams are not supported.'
            }));
            return cb();
        }
        
        if (file.isBuffer()) {
          
          var self = this;
          
          var $ = cheerio.load(file.contents.toString('utf8')),
              $svg = $('svg'),
              circleColorClasses = ['', ' circle-col-1', ' circle-col-2', ' circle-col-3', ' circle-col-4', ' circle-col-5', ' circle-col-6', ' circle-col-7'],
              iconColorClasses = ['', ' icon-col-1', ' icon-col-2'],
              iconSizeClasses = [' icon-xxs', ' icon-xs', ' icon-s', ' icon-m', ' icon-l', ' icon-xl', ' icon-xxl'],
              circleSizeClasses = [' circle-xxs', ' circle-xs', ' circle-s', ' circle-m', ' circle-l', ' circle-xl', ' circle-xxl'],
              svgData = [],
              tmplData = {
                svgCategories: []
              };
              
          // Loop through the svg and locate each symbol
          // Push each symbol id to svgData array
          $svg.find('symbol').each(function(i, elem) {
            var id = $(this).attr('id');
            svgData.push({id : id});
          });
          
          //Create each SVG category to be rendered
          createCategory({
            categoryLabel: 'Full colour icons',
            category: 'full',
            icon: {
              colorClass: '',
              sizeClass: ''
            },
            circle: {
              show: false
            }
          });
          
          createCategory({
            categoryLabel: 'Different sizes',
            category: 'full',
            icon: {
              colorClass: '',
              sizeClass: 'random'
            },
            circle: {
              show: false
            }
          });
          
          createCategory({
            categoryLabel: 'Circle and colour options',
            category: 'full',
            icon: {
              colorClass: '',
              sizeClass: ''
            },
            circle: {
              show: true,
              colorClass: 'random',
              sizeClass: '',
            }
          });
          
          createCategory({
            categoryLabel: 'Circle sizes',
            category: 'full',
            icon: {
              colorClass: '',
              sizeClass: ''
            },
            circle: {
              show: true,
              colorClass: '',
              sizeClass: 'random',
            }
          });
          
          createCategory({
            categoryLabel: 'Flat icons',
            category: 'flat',
            icon: {
              colorClass: '',
              sizeClass: ''
            },
            circle: {
              show: false,
              colorClass: '',
              sizeClass: '',
            }
          });
          
          createCategory({
            categoryLabel: 'Flat circles',
            category: 'flat',
            icon: {
              colorClass: iconColorClasses[1],
              sizeClass: 'default'
            },
            circle: {
              show: true,
              colorClass: 'random',
              sizeClass: '',
            }
          });
          
          function createCategory(opts) {
            var iconColorClassCount = 0,
                iconSizeClassCount = 0,
                circleColorClassCount = 0,
                circleSizeClassCount = 0,
                filteredSVGData;
                
            var defaults = {
              categoryLabel: '',
              category: 'all', //all, flat, full
              icon: {
                colorClass: '', //random
                sizeClass: ''
              },
              circle: {
                show: false,
                colorClass: '', //random
                sizeClass: '',
              } 
            };
            
            var s = _.merge(defaults, opts);
            var categoryData = {categoryLabel: s.categoryLabel, svgs: []}
            var circleIdx = 0;
            
            filteredSVGData = svgData.map(function(svg, idx) {
              var checkType = true,
                  sClone = _.cloneDeep(s); // Clone the settings for modification
              
              // Add the svg id and path
              sClone.icon.id = svg.id;
              sClone.icon.path = pluginOptions.spritePath;
              
              // Filter svgs by type
              if(s.category === 'flat')
                checkType = svg.id.indexOf('flat-') > -1;
              if(s.category === 'full')
                checkType = svg.id.indexOf('flat') === -1;
              if(!checkType)
                return;
              
              // Generates icon color class
              if(s.icon.colorClass === 'random') {
                if(iconColorClassCount === iconColorClasses.length)
                      iconColorClassCount = 0;
                sClone.icon.colorClass = iconColorClasses[iconColorClassCount];
                iconColorClassCount++;
              }
              
              // Generates icon size class
              if(s.icon.sizeClass === 'random') {       
                if(iconSizeClassCount === iconSizeClasses.length)
                      iconSizeClassCount = 0;
                sClone.icon.sizeClass = iconSizeClasses[iconSizeClassCount];
                iconSizeClassCount++;
              }
              
              // Need to show circles?
              if(s.circle.show) {
                // Generates circle size class
                if(s.circle.sizeClass === 'random') {     
                  if(circleSizeClassCount === circleSizeClasses.length)
                        circleSizeClassCount = 0;
                  
                  sClone.circle.sizeClass = circleSizeClasses[circleSizeClassCount];
                  circleSizeClassCount++;
                }
                
                // Generates circle color class
                if(s.circle.colorClass === 'random') {       
                  if(circleColorClassCount === circleColorClasses.length)
                        circleColorClassCount = 0;
                  sClone.circle.colorClass = circleColorClasses[circleColorClassCount];
                  circleColorClassCount++;
                }
                
                circleIdx++;
              }
              
              return sClone;
            })
            .filter(function(svg) {
              return typeof svg !== 'undefined';
            });
            
            categoryData.svgs = filteredSVGData;
            tmplData.svgCategories.push(categoryData);
          }
          
          // Read the template file
          fs.readFile(__dirname +'/template.tpl', 'utf-8', function(err, source){
            if (err)
              return console.log(err);
            
            // Compile the template using tmplData
            var template = handlebars.compile(source),
                html = template(tmplData);
                
            // Return the html file
            file = new gutil.File({path: './'+ pluginOptions.exampleName +'.html', contents: new Buffer(html)});
            
            gotFiles(file);
          });
          
        }
        
        function gotFiles(file) {
          self.push(file);
          return cb(); 
        }
    });
}

module.exports = gulpSVGExample;